/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2014 fortiss GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/

// Based on http://stackoverflow.com/a/10048884, provided by Stackoverflow user "seand".
// (follow link for license)
package org.fortiss.tooling.base.ui.annotation.view.generic.filter;

import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Text;

/**
 * Listener to react to focus in/out events of {@link Text} controls than can originate from both
 * keyboard and mouse events.
 * 
 * @author barner
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating GREEN Hash: 2D9D76B771B3E300496882293B6B8086
 */
public abstract class TextFocusListener implements Listener {

	/** Flag if {@link Text} currently has the focus. */
	private boolean hasFocus = false;

	/** Flag required to obtain focus via mouse events. */
	private boolean hadFocusOnMousedown = false;

	/** Action to be performed on focus in events. */
	protected abstract void focusIn(Text text, Event e);

	/** Action to be performed on focus out events. */
	protected abstract void focusOut(Text text, Event e);

	/** {@inheritDoc} */
	@Override
	public void handleEvent(Event e) {
		if(!(e.widget instanceof Text)) {
			return;
		}
		Text text = (Text)e.widget;

		switch(e.type) {
			case SWT.FocusIn: {
				// Covers the case where the user focuses by keyboard.
				focusIn(text, e);

				// The case where the user focuses by mouse click is special because Eclipse,
				// for some reason, fires SWT.FocusIn before SWT.MouseDown, and on mouse down
				// it cancels the selection. So we set a variable to keep track of whether the
				// control is focused (can't rely on isFocusControl() because sometimes it's
				// wrong), and we make it asynchronous so it will get set AFTER SWT.MouseDown is
				// fired.
				text.getDisplay().asyncExec(new Runnable() {
					@Override
					public void run() {
						hasFocus = true;
					}
				});

				break;
			}
			case SWT.FocusOut: {
				hasFocus = false;
				focusOut(text, e);

				break;
			}
			case SWT.MouseDown: {
				// Set the variable which is used in SWT.MouseUp.
				hadFocusOnMousedown = hasFocus;

				break;
			}
			case SWT.MouseUp: {
				if(!hadFocusOnMousedown) {
					focusIn(text, e);
				}

				break;
			}
		}
	}
}
