/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2012 fortiss GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.graphicsGL.ui.objects;

import org.fortiss.tooling.graphicsGL.ui.camera.Camera;
import org.fortiss.tooling.graphicsGL.ui.picker.ModelObjectPicker;

/**
 * Base class for 3D view objects. View objects support two methods for rendering in view and in
 * picking mode. The default does not render the object in picking mode.
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating GREEN Hash: 96A45AF5DD67955C1D3A3BD20BB9A19D
 */
public abstract class ViewObjectBase {

	/**
	 * Renders the object.
	 * 
	 * @param camera
	 *            the current camera
	 */
	public abstract void renderObject(Camera camera);

	/**
	 * Renders the object in picking mode using the given picker. The default renders nothing.
	 * Sub-classes should override if they represent a pickable object.
	 * 
	 * @param picker
	 *            the picker requesting a picker rendering run
	 * @param c
	 *            camera
	 */
	public void renderObjectForPicking(ModelObjectPicker picker, Camera c) {
		// render nothing
	}
}
