/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2015 fortiss GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.extension;

import java.util.List;

import org.eclipse.emf.ecore.EObject;
import org.fortiss.tooling.kernel.model.constraints.ConstraintInstance;
import org.fortiss.tooling.kernel.model.constraints.IConstrained;
import org.fortiss.tooling.kernel.model.constraints.IConstraintVerificationStatus;
import org.fortiss.tooling.kernel.service.IConstraintVerificationService;
import org.fortiss.tooling.kernel.service.IConstraintVerificationService.IFix;

/**
 * Interface for constraint verifiers used by {@link IConstraintVerificationService}.
 * 
 * @author aravantinos
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating GREEN Hash: 7FB60C4B022393B2FB91D2B4ADEAA3AA
 */
public interface IConstraintVerifier {

	/**
	 * Returns the unique identifier of this constraint. Should match the corresponding
	 * IConstraintVerifierUI!
	 */
	// TODO: should no longer be needed when the service handles IDs internally
	String getID();

	/** Verify the given constraint. */
	IConstraintVerificationStatus verify(ConstraintInstance constraint);

	/**
	 * @param obj
	 *            Alters <code>obj</code> so that the contained items which should not be considered
	 *            when computing the checksum have been removed.
	 */
	void removeConstraintIrrelevantContent(EObject obj);

	/**
	 * Returns a list of possible automatic fixes for the given constraint. A constraint should be
	 * candidate to fixing if it is anything but successful or outdated.
	 */
	List<IFix> fixes(ConstraintInstance constraint, IConstraintVerificationStatus status);

	/**
	 * @param constraint
	 *            Cancels the last verification of <code>constraint</code>.
	 */
	void cancel(ConstraintInstance constraint);

	/**
	 * Adds a constraint to the provided element.
	 * 
	 * Subclasses shall return <code>null</code> if the provided element should not be constrained
	 * (whatever is the reason: the constraint verifier is only judge here).
	 */
	ConstraintInstance addConstraintIfNeeded(IConstrained constrained);
}
